from .utils import SERVERS, SERVERS_JSON, MULTI_FUNCTIONS, load_domains
from .image import load_config, valid_mandatories  # , load_modules_rougail_config
from rougail import RougailConfig, RougailConvert
from .rougail.annotator import calc_providers, calc_providers_global, calc_providers_dynamic, calc_providers_dynamic_follower, calc_providers_follower
from os.path import isfile
from json import dump as json_dump, load as json_load
#
from tiramisu import Config
from .utils import value_pprint
from rougail.utils import normalize_family
from rougail import RougailSystemdTemplate
#
#
def tiramisu_display_name(kls,
                          dyn_name: 'Base'=None,
                          suffix: str=None,
                          ) -> str:
    # FIXME
    if dyn_name is not None:
        name = kls.impl_getpath() + str(suffix)
    else:
        name = kls.impl_getpath()
    return name


async def set_values(server_name, config, datas):
    if 'values' not in datas:
        return
    server_path = normalize_family(server_name)
    for vpath, value in datas['values'].items():
        path = f'{server_path}.{vpath}'
        try:
            if isinstance(value, dict):
                for idx, val in value.items():
                    await config.option(path, int(idx)).value.set(val)
            else:
                await config.option(path).value.set(value)
        except Exception as err:
            await value_pprint(await config.value.dict(), config)
            error_msg = f'cannot configure variable {vpath} for server "{server_name}": {err}'
            raise Exception(error_msg) from err


def get_ip_from_domain(domain):
    if not domain:
        return
    hostname, domainname = domain.split('.', 1)
    return DOMAINS[domainname][1][DOMAINS[domainname][0].index(hostname)]
    return optiondescription['option_0']


ROUGAIL_NAMESPACE = 'general'
ROUGAIL_NAMESPACE_DESCRIPTION = 'Général'
FUNCTIONS = {'get_ip_from_domain': get_ip_from_domain,
             'calc_providers': calc_providers,
             'calc_providers_global': calc_providers_global,
             'calc_providers_dynamic': calc_providers_dynamic,
             'calc_providers_dynamic_follower': calc_providers_dynamic_follower,
             'calc_providers_follower': calc_providers_follower,
             }


async def templates(server_name,
                    config,
                    templates_informations,
                    just_copy=False,
                    ):
    engine = RougailSystemdTemplate(config, templates_informations)
    if just_copy:
        # for all engine to none
        ori_engines = {}
        for eng in engine.engines:
            if eng == 'none':
                continue
            ori_engines[eng] = engine.engines[eng]
            engine.engines[eng] = engine.engines['none']
    try:
        await engine.instance_files()
    except Exception as err:
        print()
        print(f'=== Configuration: {server_name} ===')
        values = await config.value.dict()
        await value_pprint(values, config)
        print(err)
        print(await config.option('general.nginx.nginx_default_http').value.get())
        raise err from err
    if just_copy:
        for eng, old_engine in ori_engines.items():
            engine.engines[eng] = old_engine


async def load(cache_file,
               cache_values,
               clean_directories=False,
               copy_manual_dir=False,
               copy_tests=False,
               hide_secret=False,
               ):
    display_name=tiramisu_display_name
    #load_zones()
#    # load images
    #FIXME useful
    module_infos = load_config(copy_manual_dir, 
                               copy_tests,
                               clean_directories,
                               )
#    modules_rougail_config = load_modules_rougail_config(module_infos)
    cfg = RougailConfig.copy()
    cfg['variable_namespace'] = ROUGAIL_NAMESPACE
    cfg['variable_namespace_description'] = ROUGAIL_NAMESPACE_DESCRIPTION
    cfg['multi_functions'] = MULTI_FUNCTIONS
    cfg['extra_annotators'] = ['risotto.rougail']
    cfg['internal_functions'] = list(FUNCTIONS.keys())
    cfg['force_convert_dyn_option_description'] = True
#        cfg['module_name'] = module_name
    functions_files = set()
    load_domains()
    for server_name, datas in SERVERS.items():
        module_info = module_infos[datas['module']]
        functions_files |= set(module_info['infos'].functions_file)
    cfg['functions_file'] = list(functions_files)
    if not isfile(cache_file):
        eolobj = RougailConvert(cfg)
        cfg['risotto_globals'] = {}
        for server_name, datas in SERVERS.items():
            module_info = module_infos[datas['module']]
            cfg['dictionaries_dir'] = module_info['infos'].dictionaries_dir
            cfg['extra_dictionaries'] = module_info['infos'].extra_dictionaries
            informations = SERVERS_JSON['servers'][server_name].get('informations')
            if informations:
                cfg['risotto_globals'][server_name] = {'global:server_name': server_name,
                                                       'global:zones_name': informations['zones_name'],
                                                       'global:zones_list': list(range(len(informations['zones_name']))),
                                                       }
                values = []
                for s_idx in cfg['risotto_globals'][server_name]['global:zones_list']:
                    if not s_idx:
                        values.append(server_name)
                    else:
                        values.append(informations['extra_domainnames'][s_idx - 1])
                cfg['risotto_globals'][server_name]['global:server_names'] = values
            else:
                cfg['risotto_globals'][server_name] = {'global:server_name': server_name}
            cfg['risotto_globals'][server_name]['global:module_name'] = datas['module']
            eolobj.load_dictionaries(path_prefix=server_name)
        tiram_obj = eolobj.save(cache_file)
    else:
        with open(cache_file) as fh:
            tiram_obj = fh.read()
    optiondescription = FUNCTIONS.copy()
    try:
        exec(tiram_obj, None, optiondescription)
    except Exception as err:
        print(tiram_obj)
        raise Exception(f'unknown error when load tiramisu object {err}') from err
    config = await Config(optiondescription['option_0'],
                          display_name=display_name,
                          )
    if not isfile(cache_values):
        await config.property.pop('validator')
        await config.property.pop('cache')
        for server_name, datas in SERVERS.items():
            await set_values(server_name, config, datas)
        await config.property.read_only()
        await config.property.add('cache')
        await valid_mandatories(config)
        with open(cache_values, 'w') as fh:
            json_dump(await config.value.exportation(), fh)
    else:
        with open(cache_values, 'r') as fh:
            await config.value.importation(json_load(fh))
        await config.property.read_only()
    return module_infos, cfg, config
