from os import  environ
from json import load
from typing import List
from ipaddress import ip_address
from toml import load as toml_load
from pprint import pprint


SETTINGS = {'config': None}
MULTI_FUNCTIONS = []
DOMAINS = {}
ZONES = {}
SERVERS_JSON = {}
SERVERS = {}
CONFIGS = {}


with open(environ.get('CONFIG_FILE', 'risotto.conf'), 'r') as fh:
    RISOTTO_CONFIG = toml_load(fh)


def _(s):
    return s


def multi_function(function):
    global MULTI_FUNCTIONS
    name = function.__name__
    if name not in MULTI_FUNCTIONS:
        MULTI_FUNCTIONS.append(name)
    return function


async def value_pprint(dico, config):
    pprint_dict = {}
    for path, value in dico.items():
        if await config.option(path).option.type() == 'password' and value:
            value = 'X' * len(value)
        pprint_dict[path] = value
    pprint(pprint_dict)


def load_zones_server():
    if 'zones' in SERVERS_JSON:
        return
    with open('servers.json', 'r') as server_fh:
        SERVERS_JSON.update(load(server_fh))


def load_zones():
    global ZONES
    if ZONES:
        return

    load_zones_server()
    ZONES.update(SERVERS_JSON['zones'])
    for server_name, server in SERVERS_JSON['servers'].items():
        if 'informations' not in server:
            continue
        server_zones = server['informations']['zones_name']
        server_extra_domainnames = server['informations'].get('extra_domainnames', [])
        if len(server_zones) > 1 and len(server_zones) != len(server_extra_domainnames) + 1:
            raise Exception(f'the server "{server_name}" has more that one zone, please set correct number of extra_domainnames ({len(server_zones) - 1} instead of {len(server_extra_domainnames)})')

        for idx, zone_name in enumerate(server_zones):
            zone_domain_name = ZONES[zone_name]['domain_name']
            if idx == 0:
                zone_server_name = server_name
            else:
                zone_server_name = server_extra_domainnames[idx - 1]
            server_domain_name = zone_server_name.split('.', 1)[1]
            if zone_domain_name and zone_domain_name != server_domain_name:
                raise Exception(f'wrong server_name "{zone_server_name}" in zone "{zone_name}" should ends with "{zone_domain_name}"')
            ZONES[zone_name].setdefault('hosts', []).append(server_name)


def load_domains():
    global DOMAINS
    if DOMAINS:
        return
    load_zones()
    for zone_name, zone in SERVERS_JSON['zones'].items():
        if 'domain_name' in zone:
            hosts = []
            ips = []
            for host in ZONES[zone_name].get('hosts', []):
                hosts.append(host.split('.', 1)[0])
                ips.append(_get_ip(host, [zone_name], 0))
            DOMAINS[zone['domain_name']] = (tuple(hosts), tuple(ips))


def _get_ip(server_name: str,
            zones_name: List[str],
            index: str,
            ) -> str:
    if server_name is None or zones_name is None:
        return
    load_zones()
    index = int(index)
    zone_name = zones_name[index]
    if zone_name not in ZONES:
        raise ValueError(f"cannot set IP in unknown zone '{zone_name}'")
    zone = ZONES[zone_name]
    if server_name not in zone['hosts']:
        raise ValueError(f"cannot set IP in unknown server '{server_name}'")
    server_index = zone['hosts'].index(server_name)
    return str(ip_address(zone['start_ip']) + server_index)
